<?php

namespace Fastrack\Utilities\Seekom;

use Fastrack\Utilities\Vault;
use GuzzleHttp\Client;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Provider\GenericProvider;
use SilverStripe\Control\Controller;
use SilverStripe\Control\HTTPRequest;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\HTTPResponse_Exception;
use SilverStripe\Core\Environment;

class SeekomController extends Controller
{

	const CSRF_TOKEN = 'state';
	const AUTH_CODE = 'code';
	const SESSION_KEY = 'seekom_oath_2_state';

	private static array $allowed_actions = ['auth'];
	private static array $url_handlers = ['auth' => 'auth'];

	private GenericProvider $provider;
	private Vault $vault;
	private SeekomCredentials $credentials;

	public function __construct()
	{
		parent::__construct();
		$this->vault = new Vault(new Client());
		$this->credentials = $this->getCredentials();
		$this->setProvider();
	}


	/**
	 * @throws HTTPResponse_Exception
	 */
	public function auth(): HTTPResponse
	{
		if (!$this->getRequest()->getVar(self::AUTH_CODE)) {
			return $this->getResponse()->redirect($this->getAuthUrl());
		} else if (!$this->isValidState()) {
			$this->httpError(400, 'Invalid state');
		} else {
			$this->acquireToken();
		}
		return $this->getResponse()->setStatusCode(200);
	}

	private function getAuthUrl(): string
	{
		$authUrl = $this->provider->getAuthorizationUrl();
		$this->getRequest()->getSession()->set(self::SESSION_KEY, $this->provider->getState());
		return $authUrl;
	}

	private function refreshToken()
	{
		$accessToken = $this->credentials->getAccessToken();
		$newAccessToken = $this->provider->getAccessToken('refresh_token', [
			'refresh_token' => $accessToken->getRefreshToken(),
		]);
		$this->credentials->setAccessToken($newAccessToken);
		$this->saveCredentials();
	}

	private function acquireToken()
	{
		$accessToken = $this->provider->getAccessToken('authorization_code', [
			'code' => $this->getRequest()->getVar(self::AUTH_CODE),
			'grant_type' => 'authorization_code',
		]);
		$this->credentials->setAccessToken($accessToken);
		$this->saveCredentials();
	}

	private function setProvider()
	{
		$collaborators = ['optionProvider' => new SeekomOptionProvider()];
		$this->provider = new GenericProvider([
			'clientId' => $this->credentials->clientId,
			'clientSecret' => $this->credentials->clientSecret,
			'redirectUri' => Environment::getEnv('SS_BASE_URL') . '/ibex/auth',
			'urlAuthorize' => Controller::join_links([$this->credentials->authUrl, '/oauth/auth']),
			'urlAccessToken' => Controller::join_links([$this->credentials->apiUrl, '/oauth/token']),
			'urlResourceOwnerDetails' => Controller::join_links([$this->credentials->apiUrl, '/oauth/resource']),
		], $collaborators);
	}

	private function isValidState(): bool
	{
		$requestState = $this->getRequest()->getVar(self::CSRF_TOKEN);
		$sessionState = $this->getRequest()->getSession()->get(self::SESSION_KEY);
		if (!$requestState || !$sessionState) {
			return false;
		}
		return $requestState === $sessionState;
	}

	private function getCredentials(): SeekomCredentials
	{
		$secret = $this->vault->getSecret('clients', SeekomCredentials::getVaultPath());
		return new SeekomCredentials($secret);
	}

	private function saveCredentials()
	{
		$this->vault->setSecret('clients', SeekomCredentials::getVaultPath(), $this->credentials->getData());
	}

}
