<?php

namespace Fastrack\Utilities\ServicesApi;

use DateTime;
use Exception;
use Fastrack\Utilities\Http;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\HandlerStack;
use GuzzleLogMiddleware\Handler\LogLevelStrategy\StatusCodeStrategy;
use GuzzleLogMiddleware\Handler\MultiRecordArrayHandler;
use GuzzleLogMiddleware\LogMiddleware;
use Psr\Log\LoggerInterface;
use Psr\Log\LogLevel;
use SilverStripe\Control\Email\Email;
use SilverStripe\Core\Environment;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\ORM\ArrayList;
use SilverStripe\SiteConfig\SiteConfig;
use SilverStripe\View\ArrayData;

class ServicesApi
{

	const DATE_FORMAT = 'Y-m-d';

	/**
	 * TODO: add middleware logging
	 *
	 * @throws Exception
	 * @return QuoteResponse[]
	 */
	public function quote(string $domain, int $clientId, QuoteRequest $request): array
	{
		$url = "$domain/api/$clientId/bookings/{$request->getPropertyId()}/quote";
		try {
			$response = Http::buildClient(true)->request('GET', $url, [
				'query' => [
					'arrive' => $request->getArrive()->format('Y-m-d'),
					'depart' => $request->getDepart()->format('Y-m-d'),
					'room-id' => $request->getRoomId(),
					'rate-id' => $request->getRateId(),
					'promo' => $request->getPromo(),
					'adults' => $request->getAdults(),
					'children' => $request->getChildren(),
					'infants' => $request->getInfants(),
					'channel' => $request->getChannel(),
				],
			]);
			return $this->buildQuoteResponse(json_decode($response->getBody()->getContents(), true));
		} catch (GuzzleException|ClientException $e) {
			if ($e->getResponse()) {
				$response = json_decode($e->getResponse()->getBody()->getContents());
				if ($response->message) throw new Exception($response->message);
			}
			throw new Exception("Could not fetch quote");
		}
	}

	/**
	 * @throws Exception
	 */
	public function book(string $domain, int $clientId, BookRequest $request): BookResponse
	{
		$url = "$domain/api/$clientId/bookings/{$request->getPropertyId()}/book";
		try {
			$response = Http::buildClient(true)->request('POST', $url, [
				'auth' => [Environment::getEnv('BOOKINGS_API_USERNAME'), Environment::getEnv('BOOKINGS_API_PASSWORD')],
				'json' => $request->buildRequest(),
			]);
			return new BookResponse(json_decode($response->getBody()->getContents(), true));
		} catch (GuzzleException|ClientException $e) {
			$message = 'Could not fetch quote';
			if ($e->getResponse()) {
				$response = json_decode($e->getResponse()->getBody()->getContents());
				if ($response && $response->message) {
					$message = $response->message;
				}
			}
			$this->sendNotificationEmail($request, $message);
			throw new Exception($message);
		}
	}

	/**
	 * @throws Exception
	 */
	public function policies(string $domain, int $clientId, $propertyId): Policies
	{
		$url = "$domain/api/$clientId/properties/{$propertyId}/policies";
		try {
			$response = Http::buildClient(false)->request('GET', $url);
			return new Policies(json_decode($response->getBody()->getContents()));
		} catch (GuzzleException|ClientException $e) {
			if ($e->getResponse()) {
				$response = json_decode($e->getResponse()->getBody()->getContents());
				if ($response && $response->message) throw new Exception($response->message);
			}
			throw new Exception("Could not fetch policies");
		}
	}

	/**
	 * @param $quotesResponse
	 * @return QuoteResponse[]
	 */
	private function buildQuoteResponse($quotesResponse): array
	{
		$quotes = [];
		foreach ($quotesResponse as $quoteResponse) {
			$quotes[] = new QuoteResponse($quoteResponse);
		}
		return $quotes;
	}

	/**
	 * @param BookRequest $request
	 * @param string $errorMessage
	 * @return void
	 */
	private function sendNotificationEmail(BookRequest $request, string $errorMessage)
	{
		$transactions = ArrayList::create();
		foreach	($request->buildRequest()['transactionItems'] as $transaction) {
			$transactions->push([
				'id' => $transaction['id'],
				'amount' => $transaction['amount'] / 100,
			]);
		}

		$email = new Email();
		$email->setHTMLTemplate('Fastrack\\Email\\BookingError');
		$email->setData(ArrayData::create([
			'BookRequest' => ArrayData::create($request->buildRequest()),
			'ErrorMessage' => $errorMessage,
			'Timestamp' => date('c'),
			'Transactions' => $transactions,
			'PaymentAmount' => $request->buildRequest()['payment']['amount'] / 100,
		]));
		$email->setSubject('Booking error');
		$email->setTo(array_map(fn($email) => trim($email),
			explode(',', SiteConfig::current_site_config()->getField('BookingApiNotificationRecipients'))));
		$email->send();
	}

}
