<?php


namespace Fastrack\Utilities\Tasks;


use Aws\Credentials\Credentials;
use Aws\Sdk;
use DateTime;
use SilverStripe\Control\HTTPRequest;
use SilverStripe\Core\Environment;
use SilverStripe\Dev\BuildTask;
use Spatie\DbDumper\Compressors\GzipCompressor;
use Spatie\DbDumper\Databases\MySql;
use Spatie\TemporaryDirectory\TemporaryDirectory;

class BackupDB extends BuildTask
{

	private static $segment = 'BackupDB';
	private $s3Client;
	private $fileName;
	private $path;
	private $period;
	private $compressor;
	private $s3Folder = 'mysql_backups/';

	public function __construct()
	{
		parent::__construct();
		$databaseName = Environment::getEnv('SS_DATABASE_NAME');

		$this->compressor = new GzipCompressor();
		$this->fileName = "mysql_$databaseName.sql." . $this->compressor->useExtension();
		$temporaryDirectory = (new TemporaryDirectory())
			->name('temp')
			->force()
			->create()
			->empty();
		$this->path = $temporaryDirectory->path('db-dumps' . DIRECTORY_SEPARATOR . $this->fileName);
		$this->period = $this->getPeriod();
		$this->s3Client = $this->getS3Client();
	}

	public function getTitle()
	{
		return 'Backup DB';
	}

	public function getDescription()
	{
		return 'Backup MySQL database to S3';
	}

	/**
	 * @param HTTPRequest $request
	 */
	public function run($request)
	{
		$this->dumpDatabase();
		$backupLocation = $this->s3Folder . $this->period . '/' . $this->fileName;
		$previousLocation = $this->s3Folder . 'previous_' .  $this->period . '/' . $this->fileName;
		$this->deleteBackup($previousLocation);
		$this->copyBackup($backupLocation, $previousLocation);
		$this->deleteBackup($backupLocation);
		$this->uploadBackup($backupLocation);
	}

	private function deleteBackup($key)
	{
		if ($this->backupExists($key)) {
			$this->s3Client->deleteObject([
				'Bucket' => Environment::getEnv('AWS_BUCKET_NAME'),
				'Key' => $key
			]);
		}
	}

	private function copyBackup($source, $copyTo)
	{
		if ($this->backupExists($source)) {
			$this->getS3Client()->copyObject([
				'Bucket' => Environment::getEnv('AWS_BUCKET_NAME'),
				'CopySource' => Environment::getEnv('AWS_BUCKET_NAME') . '/' . $source,
				'Key' => $copyTo
			]);
		}
	}

	private function uploadBackup($key)
	{
		$this->s3Client->putObject([
			'Bucket' => Environment::getEnv('AWS_BUCKET_NAME'),
			'Key' => $key,
			'SourceFile' => $this->path,
		]);
	}


	private function backupExists($key)
	{
		return $this->s3Client->doesObjectExist(Environment::getEnv('AWS_BUCKET_NAME'), $key);
	}

	private function dumpDatabase()
	{
		MySql::create()
			->setDbName(Environment::getEnv('SS_DATABASE_NAME'))
			->setUserName(Environment::getEnv('SS_DATABASE_USERNAME'))
			->setPassword(Environment::getEnv('SS_DATABASE_PASSWORD'))
			->useCompressor($this->compressor)
			->dumpToFile($this->path);
	}


	private function getS3Client()
	{
		$sdk = new Sdk([
			'version' => 'latest',
			'region' => Environment::getEnv('AWS_REGION'),
			'credentials' => new Credentials(
				Environment::getEnv('AWS_ACCESS_KEY_ID'),
				Environment::getEnv('AWS_SECRET_ACCESS_KEY')
			)
		]);
		$s3Client = $sdk->createS3();
		return $s3Client;
	}

	private function getPeriod()
	{
		$now = new DateTime('now');
		if ($now->format('d') === '01') {
			return 'month';
		} else if ($now->format('l') === 'Sunday') {
			return 'week';
		} else {
			return 'day';
		}
	}
}